#!/usr/local/bin/expect -f
# @(#) $Id: acl.sh,v 1.14 2003/04/06 19:12:19 leres Exp $ (LBL)
#
# acl.sh - talk to acld
#

set prog $argv0
set len [string length $prog]
for { set i $len } { $i > 0 } { set i [expr $i - 1] } {
	if { [string index $prog $i ] == "/" } {
		incr i
		set prog [string range $prog $i $len]
		break
	}
}

match_max 2048

set f "[string trimright $prog ".sh"].log"
if [catch {log_file -a  /usr/operator/acl2/logs/$f} err] {
	send_error "$prog: warning: $err\n"
	if [catch {log_file -a  $f} err] {
		send_error "$prog: warning: $err\n"
	}
}
log_user 0

#
# Print usage and exit
#
proc usage {} {
	global prog

	send_error "usage: $prog \[drop|restore] addr\n"
	send_error "       $prog \[drop|restore] -f file\n"
	send_error "       $prog \listacl ACL\n"
	exit 1
}

#
# Reap response
#
proc getresponse { what cookie } {
	global prog
	global comment
	global lasttime

	set response "^(\[0-9]+\.\[0-9]+) $cookie $what"
	set doingcomment 0
	set comment ""
	set status 0
	set lasttime 0
	set timeout 90
	expect {
		-re "$response\r\n" {
			set lasttime $expect_out(1,string)
			# done
		}
		-re "$response -\r\n" {
			set lasttime $expect_out(1,string)
			set doingcomment 1
			exp_continue
		}
		-re "$response-failed\r\n" {
			set lasttime $expect_out(1,string)
			set status 1
			# done
		}
		-re "$response-failed -\r\n" {
			set lasttime $expect_out(1,string)
			set doingcomment 1
			set status 1
			exp_continue
		}
		"^.\r\n" {
			if { $doingcomment == 0 } {
				send_error "$prog: unexpected end of comment\n"
				exit 1
			}
			if { $lasttime == 0 } {
				send_error "$prog: missing lasttime at EOM\n"
				exit 1
			}
			# done
		}
		-re "^(\[^\r\n]*)\r\n" {
			if { $doingcomment != 0 } {
				append comment "$expect_out(1,string)\n"
			} else {
#				send_error "$prog: JUNK \"$expect_out(1,string)\"\n"
			}
			exp_continue
		}
		timeout {
			send_error "$prog: timeout $what ($timeout secs)\n"
			exit 1
		}
		eof {
			send_error "$prog: child exited\n"
			exit 1
		}
	}
	if { $comment != "" } {
		if { $status != 0 } {
			send_error "$prog: acld error: $comment"
		} elseif { $what != "acld" } {
			send_user "$comment"
		}
	}
	return $status
}

#
# Main program
#
if { $argc < 2 } {
	usage
}

set what [lindex $argv 0]
set file ""
set addr ""
set acl ""

if { $what == "drop" || $what == "restore" } {
	if { [lindex $argv 1] == "-f" } {
		if { $argc != 3 } {
			usage
		}
		set file [lindex $argv 2]
		if [catch {open $file} fd] {
			send_error "$prog: $fd\n"
			exit 1
		}
		set fd [open $file]
	} elseif { $argc == 2 } {
		set addr [lindex $argv 1]
	} else {
		usage
	}
} elseif { $argc == 2 && $what == "listacl" } {
	set acl [lindex $argv 1]
} else {
	usage
}

send_error "$prog: [clock format [clock seconds] -format "%b %d %T"] $argv\n"

#
# Connect to the acl daemon
#
spawn socket -c 127.0.0.1 1965
if { [getresponse "acld" 0] != 0 } {
	exit 1
}
set start $lasttime

set lastcookie 0
set firstcookie 1
set outstanding 0
set estatus 0
if { $what == "listacl" } {
	#
	# list an ACL
	#
	incr lastcookie
	send -- "$what $lastcookie $acl\r"
	incr outstanding
} elseif { $file == "" } {
	#
	# Handle a single address
	#
	incr lastcookie
	send -- "$what $lastcookie $addr\r"
	incr outstanding
} else {
	#
	# Loop on addrs from the file
	#
	set maxoutstanding 5
	while {[gets $fd addr] != -1} {
		incr lastcookie
		send -- "$what $lastcookie $addr\r"
		incr outstanding
		if { $outstanding >= $maxoutstanding } {
			if { [getresponse $what $firstcookie] != 0 } {
				set estatus 1
			}
			incr firstcookie
			set outstanding [expr $outstanding - 1]
		}
	}
}

# Finish up
while { $outstanding > 0 } {
	if { [getresponse $what $firstcookie] != 0 } {
		set estatus 1
	}
	incr firstcookie
	set outstanding [expr $outstanding - 1]
}

#send_error "(start $start finish $lasttime)\n"
set delta [format "%.3f" [expr $lasttime - $start]]

send_error "$prog: (took $delta seconds)\n"

incr lastcookie
send -- "exit $lastcookie\r"
expect "^(\[0-9]+\.\[0-9]+) $lastcookie exit"

if { $estatus != 0 } {
	send_error "$prog: $what FAILED\n"
}

exit $estatus
