/* @(#) $Id: acld.h,v 1.9 2003/04/06 19:09:26 leres Exp $ (LBL) */

/* I/O buffer */
struct iobuf {
	char *buf;
	size_t size;
	size_t len;
};

#include "timer.h"
#include "client.h"

/* acld states */
enum acldstate {
	ASTATE_NOTCONNECTED = 0,
	ASTATE_CONNECTED,
	ASTATE_LOGGEDIN,
	ASTATE_SENTLOGIN,
	ASTATE_SENTLISTACL,
	ASTATE_SENTSYNC,
	ASTATE_SENTAYT,
	ASTATE_READRESPONSE,
	ASTATE_READERROR,
	ASTATE_READACL,
};

/* State */
struct state {
	enum acldstate state;		/* ASTATE_* */
	const char *cmd;		/* command sent */
	int nfds;			/* number of select file descriptors */
	int s;				/* listen socket */
	int rfd;			/* expect child read pipe */
	int wfd;			/* expect child read pipe */
	pid_t pid;
	u_int nclients;			/* next client number */
	struct timeval timeout;		/* select timeout */
	struct iobuf childiobuf;
	struct client *clients;		/* linked list of clients */
	struct client *reqclient;	/* client we did the last request for */
	struct cf *cf;
	struct timer t_ayt;		/* timer for "ayt" */
	struct timer t_login;		/* timer for next login attempt */
	struct timer t_sync;		/* timer for next sync */
};

#include "acl.h"

struct intlist {
	char *i_acl;
	char *i_name;
};

/* Request types */
enum reqtype {
	REQ_UNKNOWN = 0,
	REQ_LOGOUT,
	REQ_DROP,
	REQ_RESTORE,
	REQ_STATE,
	REQ_LISTACL,
	REQ_QUERY,
	REQ_SYNC,
	REQ_VERSION,
	REQ_TEST,
	REQ_EXIT,
	REQ_HELP,
};

/* Request states */
enum reqstate {
	RSTATE_PENDING = 0,	/* not processed yet */
	RSTATE_READCOMMENT,	/* reading client comments */
	RSTATE_READRESPONSE,	/* reading child response */
	RSTATE_CHILD,		/* waiting for child to reply */
	RSTATE_DONE		/* ok to remove from queue */
};

/* Request */
struct req {
	struct req *next;
	enum reqtype type;		/* REQ_* */
	enum reqstate state;		/* RSTATE_* */
	const char *cmd;		/* client command name */
	char *aclname;			/* REQ_LISTACL ACL name */
	u_int32_t cookie;
	struct timeval ats;		/* arrival timestamp */
	struct timeval cts;		/* completion timestamp */
	struct acl acl;
	char *comment;
};

#define LOGIN_SECS		15
#define AYT_SECS		(10*60)
#define SYNC_SECS		(30*60)
#define SELECT_SECS		(10*60)

/* Globals */
extern struct state state;
extern int debug;
extern int verbose;
extern const char *prog;
extern struct s2v astate2req[];
extern const char version[];
extern const char *configfn;

/* Functions */
void freereq(struct req *);
#ifndef HAVE_INET_ATON
#ifdef __STDC__
struct in_addr;
#endif
int inet_aton(const char *, struct in_addr *);
#endif
void intlistadd(struct cf *, const char *);
void intlistfree(struct intlist *);
void iofree(struct iobuf *);
char *iogetline(struct iobuf *);
int ioread(int, struct iobuf *);
int iowrite(int, struct iobuf *);
void nbio(int);
